@@ -7,17 +7,17 @@
 # This file is part of Hive Administration Scripts.
 #
 # Hive Administration Scripts is free software: you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
+# it under the terms of the Apache License as published by the Apache
+# Foundation, either version 2.0 of the License, or (at your option) any
+# later version.
 #
 # Hive Administration Scripts is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-# GNU General Public License for more details.
+# Apache License for more details.
 #
-# You should have received a copy of the GNU General Public License
-# along with Hive Administration Scripts. If not, see <http://www.gnu.org/licenses/>.
+# You should have received a copy of the Apache License along with
+# Hive Administration Scripts. If not, see <http://www.apache.org/licenses/>.
 
 __author__ = "Luís Martinho <lmartinho@hive.pt> & João Magalhães <joamag@hive.pt>"
 """ The author(s) of the module """
@@ -34,7 +34,7 @@ __date__ = "$LastChangedDate$"
 __copyright__ = "Copyright (c) 2008-2015 Hive Solutions Lda."
 """ The copyright for the module """
 
-__license__ = "GNU General Public License (GPL), Version 3"
+__license__ = "Apache License, Version 2.0"
 """ The license for the module """
 
 import os
@@ -47,8 +47,14 @@ import legacy
 
 import admin_scripts.extra as extra
 
-USAGE_MESSAGE = "cleanup-stylesheets-python path [-r] [-u] [-n] [-p property_name_1, property_name_2, ...] [-s rule_skip_1, rule_skip_2, ...] [-e file_extension_1, file_extension_2, ...] [-w exclusion_1, exclusion_2, ...] [-c configuration_file]"
-""" The usage message """
+USAGE_MESSAGE = "cleanup-stylesheets-python path [-r] [-u] [-n]\
+[-p property_name_1, property_name_2, ...]\
+[-s rule_skip_1, rule_skip_2, ...]\
+[-e file_extension_1, file_extension_2, ...]\
+[-w exclusion_1, exclusion_2, ...]\
+[-c configuration_file]"
+""" The usage message, that is going to be displayed
+once it's requested or an error occurs """
 
 COLOR_REGEX_VALUE = r"(.*:)(.*#)([0-9a-fA-F]+)(.*)"
 """ The color regex value """
@@ -56,8 +62,10 @@ COLOR_REGEX_VALUE = r"(.*:)(.*#)([0-9a-fA-F]+)(.*)"
 COLOR_REGEX = re.compile(COLOR_REGEX_VALUE)
 """ The color regex """
 
-VALID_COLOR_LENGTHS = [3, 6]
-""" The list of valid color declaration lengths """
+VALID_COLOR_LENGTHS = (3, 6)
+""" The list of valid color declaration lengths
+meaning that only colors of these size are allowed
+by the stylesheet analyser """
 
 AT_RULES_REGEX_VALUE = r"\@charset|\@import|\@media|\@page .*;"
 """ The at rules regex value """
@@ -83,10 +91,17 @@ PROPERTY_LINE_REGEX = re.compile(PROPERTY_LINE_REGEX_VALUE)
 PROPERTY_LINE_REPLACEMENT_VALUE = "    \g<1>: \g<2>"
 """ The property line replacement value """
 
+RULES_MAP = dict(
+    margin = ("zero_to_multiple",),
+    border = ("zero_to_none",)
+)
+""" The map that associates various property names
+with the various rules that should be applied to them """
+
 def get_property_index(property_line, property_order, line_number):
     """
     Retrieves the index for the property specified in the provided property
-    line.
+    line, this index may be used to correctly position the property.
 
     @type property_line: String
     @param property_line: The property line containing the property.
@@ -110,7 +125,7 @@ def get_property_index(property_line, property_order, line_number):
     if len(property_line_splitted) > 2:
         # warns about the extra values in the line and then returns
         # with the length of the property order
-        extra.echo("WARNING: extra values found at line %d" % line_number)
+        extra.warn("Extra values found at line %d" % line_number)
 
     # runs a second split operation that limits the number of splits
     # in the line to two so that no extra problems are raised and then
@@ -126,18 +141,17 @@ def get_property_index(property_line, property_order, line_number):
     # in case the property is not in the order
     if not property_name in property_order:
         # warns about the missing property name
-        extra.echo(
-            "WARNING: order for property %s not defined at line %d" %\
+        extra.warn(
+            "Order for property %s not defined at line %d" %\
             (property_name, line_number)
         )
 
         # uses the greatest index
         return len(property_order)
 
-    # determines the index for the property name
+    # determines the index for the property name and returns
+    # the value to the caller method
     property_index = property_order.index(property_name)
-
-    # returns the property index
     return property_index
 
 def write_lines(output_buffer, lines, windows_newline):
@@ -196,7 +210,8 @@ def process_property_lines(property_lines, line_number):
     """
 
     # processes the property lines
-    processed_property_lines = [process_property_line(property_line, line_number) for property_line in property_lines]
+    processed_property_lines = [process_property_line(property_line, line_number)\
+        for property_line in property_lines]
 
     # returns the processed property lines
     return processed_property_lines
@@ -206,12 +221,17 @@ def process_property_line(property_line, line_number):
     # is considered to be empty (warning required) logs the message
     # and then returns the line itself (no processing)
     if not property_line.strip():
-        extra.echo("WARNING: empty stylesheet property at line %s" % line_number)
+        extra.warn("Empty stylesheet property at line %s" % line_number)
         return property_line
 
     # strips the line to the right so that no newline characters
     # exist (much simpler to manage lines without newlines)
     property_line = property_line.rstrip()
+    property_line = property_line.rstrip(";")
+
+    # runs the initial process of simplifying the property by running
+    # the various rules for the current property name
+    property_line = process_rules(property_line, line_number)
 
     # ensures the property line is correctly indented and
     # the property name and value are correctly separated
@@ -234,54 +254,79 @@ def process_property_line(property_line, line_number):
     # returns the processed property line
     return property_line
 
+def rule_zero_to_multiple(name, value):
+    if not value in ("0", "0px"): return
+    return "%s: 0px 0px 0px 0px;" % name
+
+def rule_zero_to_none(name, value):
+    if not value in ("0", "0px"): return
+    return "%s: none;" % name
+
+def process_rules(property_line, line_number):
+    # unpacks the property line into name and value so that proper
+    # per name rule application is possible for the line
+    property_name, property_value = property_line.split(":", 1)
+    property_name = property_name.strip()
+    property_value = property_value.strip()
+    property_value = property_value.rstrip(";")
+
+    # tries to retrieve the set of rules (as strings) that are
+    # going to be applied to the current property and then runs
+    # the iteration cycle to apply each of them on the line
+    rules = RULES_MAP.get(property_name, [])
+    for rule in rules:
+        method_name = "rule_" + rule
+        method = globals().get(method_name, None)
+        if not method: continue
+        result = method(property_name, property_value)
+        if not result: continue
+        property_line = result
+
+    # returns the "final" processed line according to the defined
+    # set of rules for the property (name)
+    return property_line
+
 def process_color_definition(property_line, line_number):
-    # tries to match the color regex
+    # tries to match the color regex and retrieves the resulting
+    # set of groups from the match, in case there's no matching
+    # the control flow is returned immediately to the caller
     line_match = COLOR_REGEX.match(property_line)
-
-    # tries to retrieve the line match group
-    line_groups = line_match and line_match.groups() or None
-
-    # in case no line groups are defined, returns
-    # the current property line immediately
+    line_groups = line_match.groups() if line_match else None
     if not line_groups: return property_line
 
-    # unpacks the line groups
+    # unpacks the various line groups into the appropriate
+    # variables to be used in the processing of the color
     property_name, pre_color, color, post_color = line_groups
 
     try:
-        # fixes the color
+        # runs the fixing color operation and then, re-joins the
+        # various groups again into a single string (from list)
         color = fix_color(color)
-
-        # packs the line groups
         line_groups = [property_name, pre_color, color, post_color]
-
-        # assembles the line with the fixed color
         property_line = "".join(line_groups)
     except Exception as exception:
-        # converts the exception to string
+        # converts the exception to string and then
+        # prints a warning to the proper output
         exception_string = legacy.UNICODE(exception)
-
-        # prints a warning
-        extra.echo("WARNING: %s near line %d" % (exception_string, line_number))
+        extra.warn("%s near line %d" % (exception_string, line_number))
 
     # returns the property line
     return property_line
 
 def fix_color(color):
-    # computes the color length
+    # computes the color length and the runs the lowercase
+    # operation in (as it's expected)
     color_length = len(color)
-
-    # lower cases the color
     color = color.lower()
 
     # in case the color length is not in the valid range
+    # raises an exception indicating the problem
     if not color_length in VALID_COLOR_LENGTHS:
-        # raises an exception
         raise Exception("invalid color length")
 
-    # in case the color is compacted
+    # in case the color is compacted must expand the
+    # string value to double the size (as expected)
     if color_length == 3:
-        # expands the color
         color_list = [value * 2 for value in color]
         color = "".join(color_list)
 
@@ -304,19 +349,18 @@ def skip_rule(start_line, rules_skip):
     # for each of the rules to skip
     for rule_skip in rules_skip:
         # checks if the rule is to be skipped
+        # and if that's the case breaks the loop
         skip_rule = rule_skip in start_line
-
-        # in case the rule is to be skipped
-        if skip_rule:
-            # stops checking
-            break
+        if skip_rule: break
 
     # returns the skip rule value
     return skip_rule
 
 def cleanup_properties(input_buffer, windows_newline, fix_extra_newlines, property_order, rules_skip):
     """
-    Cleans up the property lines. Sorts the css properties in the file, by the specified property order.
+    Cleans up the property lines. Sorts the css properties in the file,
+    by the specified property order.
+
     Ensures that all property lines are correctly terminated with semicolon.
 
     @type input_buffer: StringBuffer
@@ -372,17 +416,17 @@ def cleanup_properties(input_buffer, windows_newline, fix_extra_newlines, proper
         # in case the line contains a the start of multiline comment
         elif "/*" in line:
             # in case the comment mode is already on
-            if comments_started:
-                # prints a warning
-                extra.echo("WARNING: found opening comment inside open comment at line %d" % line_number)
+            if comments_started: extra.warn(
+                "Found opening comment inside open comment at line %d" % line_number
+            )
 
             # increments the comments started counter
             comments_started += 1
         # in case the line contains the end of multiline comment
         elif "*/" in line:
-            if not comments_started:
-                # raises an error
-                extra.echo("ERROR: found closing comment without corresponding opening at line %d" % line_number)
+            if not comments_started: extra.error(
+                "Found closing comment without corresponding opening at line %d" % line_number
+            )
 
             # decrements the comments started counter
             comments_started -= 1
@@ -423,21 +467,18 @@ def cleanup_properties(input_buffer, windows_newline, fix_extra_newlines, proper
                 # in case the rule set does not contain any property
                 # must log an information message about the empty rule
                 if rule_started and not rule_lines:
-                    extra.echo("WARNING: empty stylesheet rule at line %d" % line_number)
+                    extra.warn("Empty stylesheet rule at line %d" % line_number)
 
                 # updates the flag to signal the rule has ended
                 rule_started = False
 
-                # property lines
+                # sorts the various property lines and the processes them
                 property_lines = sorted(rule_lines, key = get_comparison_key)
-
-                # processes the property lines
                 property_lines = process_property_lines(property_lines, line_number)
 
                 # writes the lines to the buffer, considering the windows newline
+                # and then writes the line
                 write_lines(output_buffer, property_lines, windows_newline)
-
-                # writes the line
                 write_line(output_buffer, line, windows_newline)
 
                 # writes the final newline
@@ -479,13 +520,13 @@ def cleanup_properties(input_buffer, windows_newline, fix_extra_newlines, proper
             # otherwise in case this is an extra newline
             elif not needs_newline and newlines > 1:
                 # logs a warning about this extra newline
-                extra.echo("WARNING: found extra newline at line %d" % line_number)
+                extra.warn("Found extra newline at line %d" % line_number)
 
             # disables the needs newline flag
             needs_newline = False
         else:
             # warns about the statement outside a valid rule
-            extra.echo("WARNING: found statement outside rule at line %d" % line_number)
+            extra.warn("Found statement outside rule at line %d" % line_number)
 
         # writes the line to the output buffer taking into
         # account the windows newline control flag
@@ -507,7 +548,13 @@ def cleanup_properties(input_buffer, windows_newline, fix_extra_newlines, proper
 
     return output_buffer
 
-def cleanup_stylesheets(file_path_normalized, windows_newline, fix_extra_newlines, property_order, rules_skip):
+def cleanup_stylesheets(
+    file_path_normalized,
+    windows_newline,
+    fix_extra_newlines,
+    property_order,
+    rules_skip
+):
     """
     Cleans up stylesheets. Sorts the css properties in the file, by the specified property order.
     Ensures that all property lines are correctly terminated with semicolon.
@@ -549,7 +596,7 @@ def cleanup_stylesheets(file_path_normalized, windows_newline, fix_extra_newline
         # message to be printed to the standard output, then logs
         # the complete traceback messages to the same output
         exception_string = legacy.UNICODE(exception)
-        extra.echo("ERROR: %s. Skipping file %s" % (exception_string, file_path_normalized))
+        extra.error("%s. Skipping file %s" % (exception_string, file_path_normalized))
         traceback.print_exc(file = sys.stdout)
 
         # skips writing to the file
