@@ -270,6 +270,11 @@ configurations = (
             "-ms-transition-duration",
             "-moz-transition-duration",
             "-webkit-transition-duration",
+            "transition-timing-function",
+            "-o-transition-timing-function",
+            "-ms-transition-timing-function",
+            "-moz-transition-timing-function",
+            "-webkit-transition-timing-function",
             "unicode-bidi",
             "user-select",
             "-o-user-select",
